<?php

namespace App\Http\Controllers;

use App\Models\CalificacionMensaje;
use App\Models\Categoria;
use App\Models\Dependencia;
use App\Models\ImagenMensaje;
use App\Models\LogBandeja;
use App\Models\Mapa;
use App\Models\Mensaje;
use App\Models\Rol;
use App\Models\Seguimiento;
use App\Models\Turnado;
use App\Models\User;
use Exception;
use Illuminate\Support\Facades\Auth;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Http;
use Illuminate\Support\Facades\Response;


class BandejaEntradaController extends Controller
{
    public function __construct()
    {
        $this->middleware('auth');
    }


    public function indexBandeja(){
        session("session_idMensajeAbierto", 0);

        return view('indexBandeja');

    }
    public function getMensajesEntrantes(Request $request)
    {
        //Evaluamos si la sesion expiró para liberar el mensaje seleccionado por el usuario...
        if(Auth::guest()) {
            return $this->sessionExpired();
        }


        try {
            $idRol          = Auth::user()->idRol;
            $veremergencias = Rol::find($idRol)->emergencia;

            $mensajes = Mensaje::getMensajesByIDRol($idRol, $veremergencias);

            //$mensajes = '[{"idmensaje":"100","mensaje":"Reporte de fuga de agua con mucha afluencia Reporte de fuga de agua con mucha afluencia Reporte de fuga de agua con mucha afluencia Reporte de fuga de agua con mucha afluencia Reporte de fuga de agua con mucha afluencia","numerocelular":"9615792309","fecharegistro":"25/10/2024 15:18:00","idestado":"5","folio":"REP100","imagen":"fuga.jpg","longitud":"-93.10467263826075","latitud":"16.773593715881397","nombrecompleto":"GILBERTO VAZQUEZ RAMIREZ","siglasgrupo":"TV","municipio":"TUXTLA GUTIÉRREZ","emergencia":"1","colonia":"COL. BARRIO HIDALGO","dirGoogle":"3A NORTE ORIENTE 1619 CENTRO","idgrupo":"1","idColonia":"99","idusuariolog":"0","nombreusuario":""},
            //{"idmensaje":"101","mensaje":"Bache en medio de la calle, esta muy profundo","numerocelular":"9615792309","fecharegistro":"25/10/2024 15:18:00","idestado":"5","folio":"REP101","imagen":"bache.jpg","longitud":"-93.11076661714259","latitud":"16.764060734217892","nombrecompleto":"GILBERTO VAZQUEZ RAMIREZ","siglasgrupo":"TV","municipio":"TUXTLA GUTIÉRREZ","emergencia":"1","colonia":"COL. BARRIO HIDALGO","dirGoogle":"3A NORTE ORIENTE 1619 CENTRO","idgrupo":"1","idColonia":"99","idusuariolog":"0","nombreusuario":""}]';
            //$mensajes = json_decode($mensajes, true);


            $response = array(
                'status'   => 'success',
                'msg'      => '',
                'mensajes' => $mensajes
            );
        }
        catch(Exception $ex)
        {
            $response = array(
                'status' => 'fail',
                'msg' => $ex->getMessage(),
                'mensajes' => ''
            );
        }

        return Response::json($response);
    }
    public function getUltimosLogsActualizarBandeja(Request $request)
    {
        if(Auth::guest()) {
            return $this->sessionExpired();
        }

        try {

            $_fechaUltimaActualizacion  = $request->input('fechaActualizacion');
            $logsBandeja                = LogBandeja::getUltimosLogs($_fechaUltimaActualizacion);

            $response = array(
                'status'       => 'success',
                'msg'          => '',
                'logsMensajes' => $logsBandeja
            );
        }
        catch(Exception $ex)
        {
            $response = array(
                'status'      => 'fail',
                'msg'         => $ex->getMessage(),
                'logsBandeja' => ''
            );
        }

        return Response::json($response);
    }
    public function storeRespuestaMensaje(Request $request) {

        if(Auth::guest()) {
            return $this->sessionExpired();
        }


        try {
            $exception = DB::transaction(function() use ($request) {

                $formIdMensaje              = $request->input('formIdMensaje');
                $formIdEstadoMsj            = $request->input('formIdEstadoMsj');
                $formIdDependencia          = $request->input('formIdDependencia');
                $formIdCategoria            = $request->input('formIdCategoria');
                $formTxtMensajeMas          = strval($request->input('formTxtMensajeMas'));
                $formTxtRespuesta           = strval($request->input('formTxtRespuesta'));


                $withFoto                   = (int)$request->input('withFoto');
                $withKeyword                = (int)$request->input('withKeyword');
                $withDescription            = (int)$request->input('withDescription');
                $withDireccionText          = (int)$request->input('withDireccionText');
                $withExactlyDirectionMap    = (int)$request->input('withExactlyDirectionMap');
                $isRelevant                 = (int)$request->input('isRelevant');
                $formCalificacionTotal      = (int)$request->input('formCalificacionTotal');


                $formlatitud                = $request->input('formlatitud');
                $formlongitud               = $request->input('formlongitud');
                $formidcolonia              = $request->input('formidcolonia');
                $formidcalle1               = $request->input('formidcalle1');
                $formidcalle2               = $request->input('formidcalle2');
                $formidcalle3               = $request->input('formidcalle3');
                $formcalle1                 = $request->input('formcalle1');
                $formcalle2                 = $request->input('formcalle2');
                $formcalle3                 = $request->input('formcalle3');


                $mensaje                    = Mensaje::find($formIdMensaje);
                $mensaje->idEstadoMensaje   = 2;
                $mensaje->relevante         = $isRelevant;
                $mensaje->calificacion      = $formCalificacionTotal;
                $mensaje->save();


                $mapa                       = Mapa::where('idMensaje',$formIdMensaje)->first();
                $mapa->latitud              = $formlatitud;
                $mapa->longitud             = $formlongitud;
                $mapa->idCalle1             = $formidcalle1;
                $mapa->idCalle2             = $formidcalle2;
                $mapa->idCalle3             = $formidcalle2;
                $mapa->save();


                $calificacion                       = new CalificacionMensaje();
                $calificacion->idMensaje            = $formIdMensaje;
                $calificacion->withFoto             = $withFoto;
                $calificacion->withPalabraClave     = $withKeyword;
                $calificacion->withDescripcion      = $withDescription;
                $calificacion->withDireccionTexto   = $withDireccionText;
                $calificacion->withUbicacionExacta  = $withExactlyDirectionMap;
                $calificacion->eliminado            = 0;
                $calificacion->save();


                $turnado = new Turnado();
                $turnado->idMensaje                 = $formIdMensaje;
                $turnado->idDependencia             = $formIdDependencia;
                $turnado->idCategoria               = $formIdCategoria;
                $turnado->fechaTurnado              = date('Y-m-d H:i:s');
                $turnado->corregido                 = 0;
                $turnado->eliminado                 = 0;
                $turnado->save();


                $seguimiento = new Seguimiento();
                $seguimiento->idMensaje             = $formIdMensaje;
                $seguimiento->idEstadoMensaje       = 2;
                $seguimiento->idUsuario             = Auth::user()->idUsuario;
                $seguimiento->fechaRegistro         = date('Y-m-d H:i:s');
                $seguimiento->descripcion           = $formTxtRespuesta;
                $seguimiento->observaciones         = $formTxtMensajeMas;
                $seguimiento->save();


                $logMensaje = new LogBandeja();
                $logMensaje->idGrupo                = $mensaje->idGrupo;
                $logMensaje->idEstadoMensaje        = 2;
                $logMensaje->idUsuarioLog           = 0;
                $logMensaje->idMensaje              = $formIdMensaje;
                $logMensaje->fechaRegistro          = date('Y-m-d H:i:s');
                $logMensaje->mensajeJson            = '-';
                $logMensaje->jsonDisponible         = '-';
                $logMensaje->save();


                //$sendPushVigilante = $request->input('sendPushRespuestaVigilante');
                $sendPushVigilante = $request->sendPushRespuestaVigilante == null ? false : true;
                if($sendPushVigilante)
                {
                    $this->sendPushVigilante($formTxtRespuesta, $mensaje->idVigilante);
                }

            });

            if (is_null($exception)) {
                $response = array(
                    'status' => 'success',
                    'msg'    => ''
                );
            }
            else {
                throw new Exception;
            }

        }
        catch(Exception $ex)
        {
            $response = array(
                'status' => 'fail',
                'msg'    => $ex->getMessage(),
            );
        }

        return Response::json($response);
    }
    public function getDependenciasGrupo(Request $request)
    {
        //Evaluamos si la sesion expiró para liberar el mensaje seleccionado por el usuario...
        if(Auth::guest()) {
            return $this->sessionExpired();
        }

        try {
            $idGrupo      = $request->input('idGrupo');
            $dependencias = Dependencia::getDependenciasByIDGrupo($idGrupo);

            $response = array(
                'status'       => 'success',
                'msg'          => '',
                'dependencias' => $dependencias
            );
        }
        catch(Exception $ex)
        {
            $response = array(
                'status'       => 'fail',
                'msg'          => $ex->getMessage(),
                'dependencias' => ''
            );
        }

        return Response::json($response);
    }
    public function getCategoriasDependencia(Request $request)
    {
        //Evaluamos si la sesion expiró para liberar el mensaje seleccionado por el usuario...
        if(Auth::guest()) {
            return $this->sessionExpired();
        }

        try {
            $idDependencia = $request->input('idDependencia');
            $categorias    = Categoria::getCategoriasByIDDependencia($idDependencia);

            $response = array(
                'status'       => 'success',
                'msg'          => '',
                'categorias'   => $categorias
            );
        }
        catch(Exception $ex)
        {
            $response = array(
                'status'       => 'fail',
                'msg'          => $ex->getMessage(),
                'categorias'   => ''
            );
        }

        return Response::json($response);
    }
    public function setMensajeDisponibility(Request $request)
    {
        if(Auth::guest()) {
            return $this->sessionExpired();
        }

        try {

            $_idMensaje  = $request->input('idMensaje');
            $_tipoEvento = $request->input('tipoEvento');

            $idUsuario = 0;
            session("session_idMensajeAbierto", 0);

            if($_tipoEvento == "abrir") {
                $idUsuario = Auth::user()->idUsuario;
                session("session_idMensajeAbierto", $_idMensaje);
            }

            $result = LogBandeja::setMensajeDisponibility($_idMensaje, $idUsuario);

            $response = array(
                'status'        => 'success',
                'result'        => $result,
                'jsonreturnado' => '',
                'jsonbaja'      => ''

            );
        }
        catch(Exception $ex)
        {
            $response = array(
                'status'    => 'fail',
                'msg'       => $ex->getMessage(),
                'mensajes'  => ''
            );
        }

        return Response::json($response);
    }

    public function eliminarMensaje(Request $request){

        if(Auth::guest()) {
            return $this->sessionExpired();
        }

        try {
            $exception = DB::transaction(function() use ($request) {

                $formIdMensaje              = $request->input('formIdMensaje');
                $formTxtRespuesta           = $request->input('formTxtRespuesta');

                $mensaje                    = Mensaje::find($formIdMensaje);
                $mensaje->idEstadoMensaje   = 10;
                $mensaje->eliminado         = 1;
                $mensaje->save();

                $seguimiento = new Seguimiento();
                $seguimiento->idMensaje             = $formIdMensaje;
                $seguimiento->idEstadoMensaje       = 10;
                $seguimiento->idUsuario             = Auth::user()->idUsuario;
                $seguimiento->fechaRegistro         = date('Y-m-d H:i:s');
                $seguimiento->descripcion           = $formTxtRespuesta;
                $seguimiento->observaciones         = '';
                $seguimiento->save();


                $logMensaje = new LogBandeja();
                $logMensaje->idGrupo          = $mensaje->idGrupo;
                $logMensaje->idEstadoMensaje  = 10;
                $logMensaje->idUsuarioLog     = 0;
                $logMensaje->idMensaje        = $formIdMensaje;
                $logMensaje->fechaRegistro    = date('Y-m-d H:i:s');
                $logMensaje->mensajeJson      = '-';
                $logMensaje->jsonDisponible   = '-';
                $logMensaje->save();

            });

            if (is_null($exception)) {
                $response = array(
                    'status'        => 'success',
                    'msg'           => '',
                );
            }
            else {
                throw new Exception;
            }

        }
        catch(Exception $ex)
        {
            $response = array(
                'status'    => 'fail',
                'msg'       => $ex->getMessage(),
            );
        }

        return Response::json($response);
}
    public function sendPushVigilante($mensaje, $idvigilante){
        try {
            $data_viewer = Http::post('https://dwvozciudadana.misionmed.com/sendpush/allvigilantes', [
                'mensaje'       => $mensaje,
                'idsVigilantes' => $idvigilante
            ])
            ->object();

            return 'OK';
        }
        catch(\Exception $e) {
            return 'FAIL';
        }
    }
    public function testPush() {

        $data_viewer = Http::post('https://dwvozciudadana.misionmed.com/sendpush/allvigilantes', [
            'mensaje'       => 'HOLA...',
            'idsVigilantes' => 4
        ])
        ->object();

        if($data_viewer->status == 'success') {
            return 'OK';
        }
        else {
            return 'FAIL';
        }
    }

    public function sessionExpired() {

        $idUsuario = 0;
        $idMensaje = session('session_idMensajeAbierto');

        if(!empty($idMensaje)){
            LogBandeja::setMensajeDisponibility($idMensaje,$idUsuario);
        }

        return Response::json("SesionExpired");
    }
    public function storeMensajeTmp() {

        $mensaje = new Mensaje();
        $mensaje->idVigilante        = 1;
        $mensaje->idEstadoMensaje    = 1;
        $mensaje->idGrupo            = 1;
        $mensaje->numeroCelular      = '9615792309';
        $mensaje->mensaje            = 'Calle inundada los vecinos solicitan el apoyo de las autoridades DANIEL';
        $mensaje->fechaRegistro      = date('Y-m-d H:i:s');
        $mensaje->folio              = date('ymdHis-u');
        $mensaje->emergencia         = 0;
        $mensaje->calificacion       = 0;
        $mensaje->conclusionPositiva = 0;
        $mensaje->idMensajePrueba    = 0;
        $mensaje->seImprime          = 0;
        $mensaje->relevante          = 0;
        $mensaje->eliminado          = 0;
        $mensaje->save();


        $mapa   = new Mapa();
        $mapa->idMensaje        = $mensaje->idMensaje;
        $mapa->latitud          = '16.73745460';
        $mapa->longitud         = '-93.15672936';
        $mapa->altitud          = '0';
        $mapa->mapa             = 'sinubicacion.jpg';
        $mapa->direccionGoogle  = '-';
        $mapa->theGeom          = null;
        $mapa->idCalle1         = 0;
        $mapa->idCalle2         = 0;
        $mapa->idCalle3         = 0;
        $mapa->save();


        $imagen = new ImagenMensaje();
        $imagen->idMensaje = $mensaje->idMensaje;
        $imagen->imagen    = 'calleinundada01.png';
        $imagen->save();


        $mensajeJson = Mensaje::getMensajesByIDMensajeForBandeja($mensaje->idMensaje);

        $logBandeja = new LogBandeja();
        $logBandeja->idGrupo         = 1;
        $logBandeja->idEstadoMensaje = 1;
        $logBandeja->idUsuarioLog    = 0;
        $logBandeja->idMensaje       = $mensaje->idMensaje;
        $logBandeja->fechaRegistro   = date('Y-m-d H:i:s');
        $logBandeja->mensajeJson     = '['.json_encode($mensajeJson).']';
        $logBandeja->jsonDisponible  = '-';
        $logBandeja->save();

        echo 'OK-'.date('Y-m-d H:i:s');
    }
}
