<?php

namespace App\Http\Controllers;

use App\Models\Rol;
use App\Models\User;
use Exception;
use Illuminate\Http\Request;

class UsuarioController extends Controller
{
    public function __construct()
    {
        $this->middleware('auth');
    }

    public function index(Request $request){

        $txtBuscar = $request->get('txtBuscar');

        if(!empty($txtBuscar)){
            $usuarios = User::getAllUsuariosLike($txtBuscar);
        }
        else
        {
            $usuarios    = User::getAllUsuarios();
        }

        return view('usuarios.index',compact('usuarios','txtBuscar'));
    }

    public function create(){

        $rolesList = Rol::where('eliminado',0)->pluck('rol','idRol');

        return view('usuarios.create',compact('rolesList'));
    }

    public function store(Request $request){
        try {

            $usuario = new User();
            $usuario->idRol                     = $request->input('idRol');
            $usuario->nombre                    = $request->input('nombre');
            $usuario->apellidoPaterno           = $request->input('apellidoPaterno');
            $usuario->apellidoMaterno           = $request->input('apellidoMaterno');
            $usuario->email                     = $request->input('email');

            $usuario->password                  = bcrypt($request->input('password'));
            $usuario->passwordVisible           = $request->input('password');
            $usuario->eliminado                 = 0;
            $usuario->save();

            return redirect('usuarios')->with('status_success', 'Usuario creado Satisfactoriamente.');
        }
        catch (Exception $e){
            return redirect('usuarios/create')->with('status_fail', 'Error: '.$e->getMessage());
        }
    }

    public function edit($idUsuario){

        $rolesList = Rol::where('eliminado',0)->pluck('rol','idRol');
        $usuario    = User::find($idUsuario);

        return view('usuarios.edit',compact('rolesList','usuario'));
    }

    public function update(Request $request, $idUsuario){

        try {

            $usuario = User::find($idUsuario);
            $usuario->idRol                     = $request->input('idRol');
            $usuario->nombre                    = $request->input('nombre');
            $usuario->apellidoPaterno           = $request->input('apellidoPaterno');
            $usuario->apellidoMaterno           = $request->input('apellidoMaterno');
            $usuario->email                     = $request->input('email');
            $usuario->eliminado                 = 0;
            $usuario->save();

            return redirect('usuarios')->with('status_success', 'Usuario Actualizado Satisfactoriamente.');
        }
        catch (Exception $e){
            return redirect('usuarios/'.$idUsuario.'/edit')->with('status_fail', 'Error: '.$e->getMessage());
        }
    }

    public function edit_password($idUsuario)
    {
        $usuario    = User::find($idUsuario);

        return view('usuarios.edit_password',compact('usuario'));
    }

    public function update_password(Request $request, $idUsuario){

        try {

            $usuario = User::find($idUsuario);
            $usuario->password                  = bcrypt($request->input('password'));
            $usuario->passwordVisible           = $request->input('password');
            $usuario->save();

            return redirect('usuarios')->with('status_success', 'Usuario Actualizado Satisfactoriamente.');
        }
        catch (Exception $e){
            return redirect('usuarios/'.$idUsuario.'/edit')->with('status_fail', 'Error: '.$e->getMessage());
        }
    }


    public function destroy($idUsuario){
        try {

            $usuario = User::find($idUsuario);
            $usuario->eliminado           = 1;
            $usuario->save();

            return redirect('usuarios')->with('status_success', 'Usuario Eliminado Satisfactoriamente.');
        }
        catch (Exception $e){
            return redirect('usuarios/'.$idUsuario.'/edit')->with('status_fail', 'Error: '.$e->getMessage());
        }
    }
}
