<?php

namespace App\Models;

use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Support\Facades\DB;

class Cliente extends Model
{
    use HasFactory;

    protected $table = 'tbl_clientes';
    protected $primaryKey = 'idCliente';

    protected $fillable = [
        'nombre',
        'apellidoPaterno',
        'apellidoMaterno',
        'email',
        'celular',
        'fechaNacimiento',
        'eliminado'
    ];


    public static function getAllClientes(){

        $clientes = DB::table('tbl_clientes')
            ->select(
                'tbl_clientes.idCliente',
                'tbl_clientes.nombre',
                'tbl_clientes.apellidoPaterno',
                'tbl_clientes.apellidoMaterno',
                'tbl_clientes.email',
                'tbl_clientes.celular',
                'tbl_clientes.fechaNacimiento',
            )
            ->where('tbl_clientes.eliminado',  0)
            ->orderBy('tbl_clientes.idCliente','desc')
            ->paginate(30);


        return $clientes;

    }
    public static function getAllClientesLike($txtbuscar)
    {
        $clientes = DB::table('tbl_clientes')
            ->select(
                'tbl_clientes.idCliente',
                'tbl_clientes.nombre',
                'tbl_clientes.apellidoPaterno',
                'tbl_clientes.apellidoMaterno',
                'tbl_clientes.email',
                'tbl_clientes.celular',
                'tbl_clientes.fechaNacimiento',
            )
            ->where(function ($query) use ($txtbuscar) {
                $query->where('tbl_clientes.nombre', "LIKE", "%$txtbuscar%")
                    ->where('tbl_clientes.eliminado',  0);
            })
            ->orWhere(function ($query) use ($txtbuscar) {
                $query->where('tbl_clientes.apellidoPaterno', "LIKE", "%$txtbuscar%")
                    ->where('tbl_clientes.eliminado', 0);
            })
            ->orWhere(function ($query) use ($txtbuscar) {
                $query->where('tbl_clientes.apellidoMaterno', "LIKE", "%$txtbuscar%")
                    ->where('tbl_clientes.eliminado', 0);
            })
            ->orWhere(function ($query) use ($txtbuscar) {
                $query->where(DB::raw("CONCAT_WS(' ',tbl_clientes.apellidoPaterno,tbl_clientes.apellidoMaterno,tbl_clientes.nombre)"), "LIKE", "%$txtbuscar%")
                    ->where('tbl_clientes.eliminado',  0);
            })
            ->orWhere(function ($query) use ($txtbuscar) {
                $query->where(DB::raw("CONCAT_WS(' ',tbl_clientes.nombre,tbl_clientes.apellidoPaterno,tbl_clientes.apellidoMaterno)"), "LIKE", "%$txtbuscar%")
                    ->where('tbl_clientes.eliminado', '=', 0);
            })
            ->orWhere(function ($query) use ($txtbuscar) {
                $query->where('tbl_clientes.email', "LIKE", "%$txtbuscar%")
                    ->where('tbl_clientes.eliminado', '=', 0);
            })
            ->orderBy('tbl_clientes.idCliente','desc')
            ->paginate(30);

        return $clientes;

    }

}
