<?php

namespace App\Models;

use Carbon\Carbon;
use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\DB;

class LogBandeja extends Model
{
    use HasFactory;

    protected $table = 'tbl_log_bandeja';
    protected $primaryKey = 'idLogBandeja';

    protected $fillable = [
    ];


    public static function getUltimosLogs($fechaUltimaActualizacion)
    {
        $fecha = Carbon::createFromFormat('d/m/Y H:i:s', $fechaUltimaActualizacion)->format('Y-m-d H:i:s');

        $data = DB::table('tbl_log_bandeja')
            ->select(
                'tbl_log_bandeja.idLogBandeja',
                'tbl_log_bandeja.idGrupo',
                'tbl_log_bandeja.idEstadoMensaje',
                'tbl_log_bandeja.idUsuarioLog',
                'tbl_log_bandeja.idMensaje',
                DB::Raw("DATE_FORMAT(tbl_log_bandeja.fechaRegistro, '%d/%m/%Y %H:%i:%s') as fechaRegistro"),
                'tbl_log_bandeja.mensajeJson',
                'tbl_log_bandeja.jsonDisponible'
            )
            ->where('fechaRegistro', '>', $fecha)
            ->orderBy('fechaRegistro', 'ASC')
            ->get();

        return $data;
    }


    public static function setMensajeDisponibility($idMensaje, $idUsuario)
    {
        //Obtenemos algunos datos...
        $mensaje                = Mensaje::find($idMensaje);
        $nombreUsuario          = Auth::user()->nombre.' '.Auth::user()->apellidoPaterno.' '.Auth::user()->apellidoMaterno;

        if($idUsuario == 0) {
            //Cierra la ventana de respuesta, osea, lo está liberando, insertamos...
            $logMensaje = new LogBandeja();
            $logMensaje->idGrupo          = $mensaje->idGrupo;
            $logMensaje->idEstadoMensaje  = $mensaje->idEstadoMensaje;
            $logMensaje->idUsuarioLog     = $idUsuario;
            $logMensaje->idMensaje        = $idMensaje;
            $logMensaje->fechaRegistro    = date('Y-m-d H:i:s');
            $logMensaje->mensajeJson      = '';
            $logMensaje->jsonDisponible   = '[{"idmensaje":"'.$idMensaje.'","idusuariolog":"'.$idUsuario.'","nombreusuario":"-"}]';
            $logMensaje->save();

            $result = 1;
        }
        else {
            //LO ESTAN ABRIENDO...
            $lastLog = DB::table('tbl_log_bandeja')->where('idMensaje', $idMensaje)->orderBy('fechaRegistro', 'DESC')->first();
            if ($lastLog)
            {
                if($lastLog->idEstadoMensaje == 6) {
                    //Está siendo atendido...
                    if ($lastLog->idUsuarioLog == $idUsuario) {
                        //NO insertamos puesto q yo lo tengo...
                        $result = 1;
                    } else {
                        //NO insertamos, lo tiene alguien más...
                        $result = 0;
                    }
                }
                else {

                    //No está siendo atendido por nadie, insertamos...
                    $logMensaje = new LogBandeja();
                    $logMensaje->idGrupo         = $mensaje->idGrupo;
                    $logMensaje->idEstadoMensaje = 6;
                    $logMensaje->idUsuarioLog    = $idUsuario;
                    $logMensaje->idMensaje       = $idMensaje;
                    $logMensaje->fechaRegistro   = date('Y-m-d H:i:s');
                    $logMensaje->mensajeJson     = '';
                    $logMensaje->jsonDisponible  = '[{"idmensaje":"'.$idMensaje.'","idusuariolog":"'.$idUsuario.'","nombreusuario":"'.$nombreUsuario.'"}]';
                    $logMensaje->save();

                    $result = 1;
                }
            }
            else {

                //No está siendo atendido por nadie, insertamos...
                $logMensaje = new LogBandeja();
                $logMensaje->idGrupo         = $mensaje->idGrupo;
                $logMensaje->idEstadoMensaje = 6;
                $logMensaje->idUsuarioLog    = $idUsuario;
                $logMensaje->idMensaje       = $idMensaje;
                $logMensaje->fechaRegistro   = date('Y-m-d H:i:s');
                $logMensaje->mensajeJson     = '';
                $logMensaje->jsonDisponible  = '[{"idmensaje":"'.$idMensaje.'","idusuariolog":"'.$idUsuario.'","nombreusuario":"'.$nombreUsuario.'"}]';;
                $logMensaje->save();

                $result = 1;
            }
        }

        return $result;
    }

}
