<?php

namespace App\Models;

use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Foundation\Auth\User as Authenticatable;
use Illuminate\Notifications\Notifiable;
use Illuminate\Support\Facades\DB;
use Laravel\Sanctum\HasApiTokens;

class User extends Authenticatable
{
    use HasApiTokens, HasFactory, Notifiable;


    protected $table = 'cat_usuarios';
    protected $primaryKey = 'idUsuario';


    protected $fillable = [
        'nombre',
        'apellidoPaterno',
        'apellidoMaterno',
        'email',
        'password',
    ];


    protected $hidden = [
        'password',
        'remember_token',
    ];


    protected $casts = [
        'email_verified_at' => 'datetime',
        'password'          => 'hashed',
    ];



    public static function SINUSARIgetAllUsuarios(){

        $usuarios = DB::table('tbl_usuarios')
            ->join('cat_roles', 'tbl_usuarios.idRol', '=', 'cat_roles.idRol')
            ->select(
                'tbl_usuarios.idUsuario',
                'tbl_usuarios.idRol',
                'cat_roles.rol',
                'tbl_usuarios.nombre',
                'tbl_usuarios.apellidoPaterno',
                'tbl_usuarios.apellidoMaterno',
                'tbl_usuarios.email',
                'tbl_usuarios.passwordVisible',
            )
            ->where('tbl_usuarios.eliminado',  0)
            ->orderBy('tbl_usuarios.idUsuario','desc')
            ->paginate(30);


        return $usuarios;

    }
    public static function SINUSARgetAllUsuariosLike($txtbuscar)
    {
        $usuarios = DB::table('tbl_usuarios')
            ->join('cat_roles', 'tbl_usuarios.idRol', '=', 'cat_roles.idRol')
            ->select(
                'tbl_usuarios.idUsuario',
                'tbl_usuarios.idRol',
                'cat_roles.rol',
                'tbl_usuarios.nombre',
                'tbl_usuarios.apellidoPaterno',
                'tbl_usuarios.apellidoMaterno',
                'tbl_usuarios.email',
                'tbl_usuarios.passwordVisible',
            )
            ->where(function ($query) use ($txtbuscar) {
                $query->where('tbl_usuarios.nombre', "LIKE", "%$txtbuscar%")
                    ->where('tbl_usuarios.eliminado',  0);
            })
            ->orWhere(function ($query) use ($txtbuscar) {
                $query->where('tbl_usuarios.apellidoPaterno', "LIKE", "%$txtbuscar%")
                    ->where('tbl_usuarios.eliminado', 0);
            })
            ->orWhere(function ($query) use ($txtbuscar) {
                $query->where('tbl_usuarios.apellidoMaterno', "LIKE", "%$txtbuscar%")
                    ->where('tbl_usuarios.eliminado', 0);
            })
            ->orWhere(function ($query) use ($txtbuscar) {
                $query->where(DB::raw("CONCAT_WS(' ',tbl_usuarios.apellidoPaterno,tbl_usuarios.apellidoMaterno,tbl_usuarios.nombre)"), "LIKE", "%$txtbuscar%")
                    ->where('tbl_usuarios.eliminado',  0);
            })
            ->orWhere(function ($query) use ($txtbuscar) {
                $query->where(DB::raw("CONCAT_WS(' ',tbl_usuarios.nombre,tbl_usuarios.apellidoPaterno,tbl_usuarios.apellidoMaterno)"), "LIKE", "%$txtbuscar%")
                    ->where('tbl_usuarios.eliminado', '=', 0);
            })
            ->orWhere(function ($query) use ($txtbuscar) {
                $query->where('tbl_usuarios.email', "LIKE", "%$txtbuscar%")
                    ->where('tbl_usuarios.eliminado', '=', 0);
            })
            ->orderBy('tbl_usuarios.idUsuario','desc')
            ->paginate(30);

        return $usuarios;

    }

}
